
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

      SUBROUTINE BARRIER(JDATE, JTIME)

C     BY: Lucas A. J. Bastien. Parts of this code (or all of it) may be
C     based on or inspired by pre-existing CMAQ and/or CMAQ adjoint
C     code, with possibly some sections copied without modification.
C
C     PURPOSE: Wait for all processors to reach this point.
C
C     NOTES: There are two versions of this routine: a serial version
C     and a parallel version.

C     ------------------------------------------------------------------

      IMPLICIT NONE

C     Include files

      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_MPICH

C     Dummy arguments

      INTEGER, INTENT(IN) :: JDATE ! current model date (YYYYDDD)
      INTEGER, INTENT(IN) :: JTIME ! current model time (HHMMSS)

C     Local variables

      CHARACTER(LEN=16) :: PNAME = "BARRIER"
      LOGICAL, SAVE     :: FIRSTIME = .TRUE.
      INTEGER, SAVE     :: LOGDEV
      INTEGER           :: MPIERR
      CHARACTER(LEN=96) :: MSG

C     ------------------------------------------------------------------

      IF ( FIRSTIME ) THEN

         FIRSTIME = .FALSE.
         LOGDEV = INIT3()

      END IF

      CALL MPI_BARRIER(MPI_COMM_WORLD, MPIERR)

      IF (MPIERR .NE. MPI_SUCCESS) THEN
         MSG = "Error in call of MPI_BARRIER"
         CALL M3EXIT(PNAME, JDATE, JTIME, MSG, XSTAT2)
      END IF

      END SUBROUTINE BARRIER
